#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <fcntl.h>
#include "kpci_pio96.h"

#define DEVICENAME "/dev/kpci/card0"
#define TEST_BYTE 0x52
#define PHYSICAL_OFFSET 0x0

int main(int argc, char *argv[])
{
    int i, fd;
    unsigned int tmp;
    unsigned char a, b, c;
    void *mem;
    Kpci_registers volatile * card;

    if ((fd = open(DEVICENAME, O_RDWR)) < 0)
    {
	fprintf(stderr, "kpci_test: error opening device %s\n", DEVICENAME);
	perror("kpci_test");
	exit(1);
    }
    mem = mmap(NULL, getpagesize(), PROT_WRITE | PROT_READ, MAP_SHARED, fd, 0);
    if (mem == MAP_FAILED) {
	fprintf(stderr, "kpci_test: mmap failed\n");
	perror("kpci_test");
	exit(2);
    }
    card = (Kpci_registers *)(mem + PHYSICAL_OFFSET);
    printf("I/O card ports are mapped at 0x%x\n\n", (unsigned int)card);
    fflush(stdout);

    /* Configure all ports except the last one for input */
    for (i=0; i<KPCI_PORTGROUPS-1; i++)
	card->ctrl[i] = 0;

    /* Configure port A on the last port group for output */
    card->ctrl[KPCI_PORTGROUPS-1] = KPCI_PORTA_OUT;

    /* Write something to port A on the last port group */
    card->port[KPCI_PORTGROUPS-1] = kpci_toport(TEST_BYTE, 0, 0);

    /* Read the data from all ports. Port access is slow,
     * so we minimize the number of port readings by using
     * a temporary variable.
     */
    for (i=0; i<KPCI_PORTGROUPS; i++) {
	tmp = card->port[i];
 	kpci_fromport(tmp, a, b, c);
	printf("Port group %d reads 0x%08x,  PA%d = 0x%02x, PB%d = 0x%02x, PC%d = 0x%02x\n",
	       i, tmp, i, a, i, b, i, c);
    }

    /* Configure the last port group for input, just to make
       sure that all ports are in high-impedance state */
    card->ctrl[KPCI_PORTGROUPS-1] = 0;

    /* Check the last port readback */
    if (a == TEST_BYTE)
	printf("\nSuccess\n");
    else
	printf("\nFailure\n");

    exit(0);
}
